<?php
namespace Teknisa\SUP\Modules\Cadastros\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class MarcaAcoesSelectController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $connection;

	public function __construct($crudService, ParameterBag $parameterBag, $connection, $environment, $entityManager) {
        $this->crudService   = $crudService;
        $this->parameterBag  = $parameterBag;
        $this->con           = $connection;
        $this->environment   = $environment;
        $this->entityManager = $entityManager;
    }
    
    private function strPadRigthProd($value){
        return str_pad($value, 20, 'Z', STR_PAD_RIGHT);
    }
    // Código relacionado ao cadastramento automático
    
    public function cadastroAutomaticoSelect(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        //var_dump($row);die;
        try{
            $parameter = array(
                'P_CDFILIAL'        => $this->environment->getCdFilial(),
                'P_CDMARCA'         => $row['CDMARCA'],
                'P_CDARVPRODINI'    => $row['CDARVPRODINI'],
                'P_CDARVPRODFIN'    => $row['CDARVPRODFIN'] = self::strPadRigthProd($row['CDARVPRODFIN'])
            );
            
            $query = "SELECT PRODUTO.CDPRODUTO,
                             PRODUTO.CDARVPROD,
                            :P_CDMARCA AS CDMARCA
                        FROM 
                            (SELECT PRODUTO.*,
                                 'SEG' AS IDTPPRODUTO
                            FROM NIVELPRODSEG, PRODUTO, FILIAL, PRODUTO P
                            WHERE FILIAL.CDSEGMER = NIVELPRODSEG.CDSEGMER
                                    AND FILIAL.CDFILIAL = :P_CDFILIAL
                                    AND NIVELPRODSEG.CDPRODUTO = P.CDPRODUTO
                                    AND PRODUTO.CDARVPROD LIKE P.CDARVPROD || '%'
                                    AND NOT EXISTS 
                                (SELECT CDPRODUTO
                                FROM ROTFABRICA
                                WHERE ROTFABRICA.CDPRODUTO = PRODUTO.CDPRODUTO)
                                        AND NOT EXISTS 
                                    (SELECT CDPRODUTO
                                    FROM ROTFABLOCAL
                                    WHERE CDFILIAL = :P_CDFILIAL
                                            AND ROTFABLOCAL.CDPRODUTO = PRODUTO.CDPRODUTO)
                                    UNION
                                    ALL SELECT PRODUTO.*,
                                 'PROD' AS IDTPPRODUTO
                                    FROM FILIAL, PRODUTO
                                    WHERE FILIAL.CDSEGMER IS NULL
                                            AND FILIAL.CDFILIAL = :P_CDFILIAL
                                            AND NOT EXISTS 
                                        (SELECT CDPRODUTO
                                        FROM ROTFABRICA
                                        WHERE ROTFABRICA.CDPRODUTO = PRODUTO.CDPRODUTO)
                                                AND NOT EXISTS 
                                            (SELECT CDPRODUTO
                                            FROM ROTFABLOCAL
                                            WHERE CDFILIAL = :P_CDFILIAL
                                                    AND ROTFABLOCAL.CDPRODUTO = PRODUTO.CDPRODUTO)) PRODUTO
                                        WHERE PRODUTO.CDARVPROD
                                        BETWEEN :P_CDARVPRODINI
                                        AND :P_CDARVPRODFIN
                                    AND PRODUTO.CDPRODESTO IS NOT NULL
                                AND NOT EXISTS 
                            (SELECT MARCPROD.CDMARCA
                            FROM MARCPROD, PRODUTO P
                            WHERE MARCPROD.CDPRODUTO = P.CDPRODUTO
                                    AND MARCPROD.CDMARCA = :P_CDMARCA
                                    AND PRODUTO.CDARVPROD LIKE P.CDARVPROD || '%')";
            $stmt = $this->con->prepare($query);
            $stmt->execute($parameter);
            $select = $stmt->fetchAll();
            $this->acoesInsert($select, $response);
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
		 
    }

    public function acoesInsert($select, $response){
        foreach($select as $row){
            try{
                $parameter = array(
                    'P_CDPRODUTO' => $row['CDPRODUTO'],
                    'P_CDMARCA'   => $row['CDMARCA']
                );
                
                $query = "INSERT INTO MARCPROD (CDPRODUTO, CDMARCA) VALUES (:P_CDPRODUTO, :P_CDMARCA)";
                $stmt = $this->con->prepare($query);
                $stmt->execute($parameter);
                $this->selectAdqVdProduto($row, $response);
            }catch(\Exception $e){
                $response->setError(new DTO\Response\Error($e->getMessage(), 1));
            }
        }
		 
    }
    
    public function selectAdqVdProduto($row, $response){
        try{
            $parameter = array('P_CDARVPROD' => $row['CDARVPROD']);
            $query = "SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, SGUNIDADE FROM PRODUTO WHERE CDARVPROD = :P_CDARVPROD";
            $stmt = $this->con->prepare($query);
            $stmt->execute($parameter);
            $adqVdProd = $stmt->fetchAll()[0];
            $this->acoesProcedure($row, $adqVdProd, $response);
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function acoesProcedure($row, $adqVdProd, $response){
        try{
            $connection = $this->entityManager->getConnection();
            $procedure = new StoredProcedure($connection, 'INSERE_PRODFORNCMP_MARCFORNCMP');
            $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, '', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDMARCA',    Param::PARAM_INPUT, $row['CDMARCA'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPRODUTO',  Param::PARAM_INPUT, $adqVdProd['CDPRODUTO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDARVPROD',  Param::PARAM_INPUT, $adqVdProd['CDARVPROD'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_ARRED',      Param::PARAM_INPUT, '', Param::PARAM_TYPE_INT));
            $procedure->addParam(new Param('P_CDFORNORIG', Param::PARAM_INPUT, '', Param::PARAM_TYPE_STR));
            $procedure->execute();
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
		 
    }
    
    // Código relacionado a exportação de produto por marca
    
    public function exportacaoMarcaSelect(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        try {
            $query = "SELECT MARCA.CDMARCA,
                            MARCPROD.CDPRODUTO,
                            PRODUTO.CDARVPROD
                        FROM MARCPROD,
                            MARCA,
                            PRODUTO
                        WHERE MARCPROD.CDPRODUTO = PRODUTO.CDPRODUTO
                          AND MARCPROD.CDMARCA = :P_CDMARCAORG
                          AND (INSTR(:P_CDMARCADES, MARCA.CDMARCA) <> 0
                               OR :P_TCDMARCADES = 'T')
                          AND (INSTR(:P_CDARVPROD, PRODUTO.CDARVPROD) <> 0
                               OR :P_TCDARVPROD = 'T')
                          AND NOT EXISTS
                            (SELECT A.CDPRODUTO
                             FROM MARCPROD A,
                                  PRODUTO P
                             WHERE A.CDPRODUTO = P.CDPRODUTO
                               AND (P.CDARVPROD LIKE PRODUTO.CDARVPROD || '%'
                                    OR PRODUTO.CDARVPROD LIKE P.CDARVPROD || '%')
                               AND A.CDMARCA = MARCA.CDMARCA)";
                               
            $parameter = array(
                'P_CDMARCADES'  => $row['CDMARCADES'],
                'P_TCDMARCADES' => $row['TCDMARCADES'],
                'P_CDARVPROD'   => $row['CDARVPROD'],
                'P_TCDARVPROD'  => $row['TCDARVPROD'],
                'P_CDMARCAORG'  => $row['CDMARCAORI']
            );
            $stmt = $this->con->prepare($query);
            $stmt->execute($parameter);
            $select = $stmt->fetchAll();
            $this->acoesInsert($select, $response);
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
		 
    }
}