<?php
namespace Teknisa\SUP\Modules\Cadastros\Services;

use Teknisa\SUP\Util\SupUtilities;
use Teknisa\Libs\Util\DateUtil;

class ConfigAlcadasService
{
    const NOME_PARAMETROS_CONFIG_ALCADA = array(
        'NRCONFIGALCADA',
        'NMALCADA',
        'IDLANCAFEXTRA',
        'IDLANCSCEXTRA',
        'IDTIPOAF',
        'IDTIPOSC',
        'IDTPLANC',
        'IDCANCEL',
        'IDBAIXMANUAL',
        'IDFORAORC',
        'IDEXCLUSAO',
        'IDIMPGESTFIL',
        'VRPLANEJAMENTO',
        'VRSUPERVISOR',
        'VRGERREGIONAL',
        'VRDIRETORIA',
        'NRORG',
        'NRORGINCLUSAO',
        'CDOPERINCLUSAO',
        'NRORGULTATU',
        'CDOPERULTATU',
        'IDTIPORCAOP',
        'CDDESPPREV',
        'IDTPALCADA',
        'IDEXIGETAPROVE',
        'IDVALIDAALTER',
        'CDTPCTPAGAR',
        'CDSETOR',
        'IDALTERACAO',        
        'IDTIPOVRAPROV',
    );
    
    const DATAS_COM_SYSDATE_INSERT = array(
        'DTULTATUALC',
        'DTINCLUSAO',
        'DTULTATU',
    );
    
    const DATAS_COM_SYSDATE_UPDATE = array(
        'DTULTATU',
    );
    
    public function __construct(SupUtilities $supUtilities, $environment)
    {
        $this->supUtilities = $supUtilities;
        $this->environment = $environment;
    }

    /********************
     *     INSERTS      *
     * ******************/
    
    public function saveConfigAlcada($row)
    {
        try {
            $parameters = self::criarParametrosConfiguracaoAlcada($row);
            
            $columnsNames = self::getNomeColunasInsertFormatado($parameters);
            $inputParamsName = self::getValorParametrosInsertFormatado($parameters);
            
            $sql = "INSERT INTO CONFIGALCADA ( " . $columnsNames . " ) VALUES ( " . $inputParamsName . " )";
            
            $parameters = self::removeSysdate($parameters, self::DATAS_COM_SYSDATE_INSERT);
            $parameters = self::removeDefault($parameters);
            
            $this->supUtilities->crud($sql, $parameters);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * @saveLogAlteracao
     * Funcionalidade incompleta. Aguardando criação da tabela de log para continuação do processo.
     */
    // public function saveLogAlteracao($numeroConfigAlcada, $nomePropriedade, $fraseDeAlteracao)
    // {
    //     try {
    //         $sql = "";
            
    //         $parameters = array(
    //             'NRCONFIGALCADA' => $numeroConfigAlcada,
    //             'NMPROPRIEDADE'  => $nomePropriedade,
    //             'DSPROPRIEDADE'  => $fraseDeAlteracao,
    //             // 'DTINCLUSAO'  => ''; // SYSDATE na query
    //             'CDOPERINCLUSAO' => $this->environment->getCdOperador(),
    //             'NRORGINCLUSAO'  => $this->environment->getNrOrg(),
    //             'NRORG'          => $this->environment->getNrOrg(),
    //         );
            
    //         $this->supUtilities->supUtilities($sql, $parameters);
    //     } catch (\Exception $e) {
    //         throw $e;
    //     }
    // }
    
    /**
     * Substituição da função saveLogAlteracao até que a sua funcionalidade seja implementada.
     */
    // public function saveLogAlteracaoArquivo($numeroConfigAlcada, $propriedadesAlteradas)
    // {
    //     $dataAtual = DateUtil::getDataAtual();
    //     $diretorio = self::getDiretorioLog();
    //     $nomeArquivo = self::getNomeArquivo($dataAtual, $numeroConfigAlcada);
    //     $arquivoLog = fopen(self::getCaminhoCompletoArquivoLog($diretorio, $nomeArquivo), "a");
        
    //     foreach ($propriedadesAlteradas as $nomePropriedade => $fraseDeAlteracao) {
    //         fwrite($arquivoLog, self::getLinhaLogFormatada($dataAtual, $nomePropriedade, $fraseDeAlteracao));
    //     }
        
    //     fclose($arquivoLog);
    // }
    
    /********************
     *     UPDATES      *
     * ******************/
     
    public function updateConfigAlcada($row)
    {
        $isOperacaoInclusao = false;
        $parameters = self::criarParametrosConfiguracaoAlcada($row, $isOperacaoInclusao);
        $colunasAndValor = self::getColunaAndValorParametrosUpdateFormatado($parameters);

        $sql = "UPDATE CONFIGALCADA SET " . $colunasAndValor . " WHERE NRCONFIGALCADA = :NRCONFIGALCADA";
        
        $parameters['NRCONFIGALCADA'] = $row['NRCONFIGALCADA'];
        $parameters = self::removeSysdate($parameters, self::DATAS_COM_SYSDATE_UPDATE);
        $parameters = self::removeDefault($parameters);
        
        try {
            $this->supUtilities->crud($sql, $parameters);
        } catch (Exception $e) {
            throw new \Exception('Erro ao editar a Alçada!');
        }
    }
    
    /********************
     *     DELETES      *
     * ******************/
     
    public function deleteConfigAlcada($row)
    {
        $sql = "DELETE FROM CONFIGALCADA WHERE NRCONFIGALCADA = :NRCONFIGALCADA";
        $params = array(
            'NRCONFIGALCADA' => $row['NRCONFIGALCADA'],
        );
        
        try {
            $this->supUtilities->crud($sql, $params);
        } catch (Exception $e) {
            throw new \Exception('Erro ao deletar Alçada!');
        }
    }
    
    /********************
     *      FINDS       *
     ********************/
     
    public function findAlcada($NRCONFIGALCADA)
    {
        $sql = "SELECT C.*, ( SELECT COUNT(*) FROM APROVALCADA WHERE NRCONFIGALCADA = C.NRCONFIGALCADA AND IDAPROVADO = 'PENDENTE' ) APROVACAO FROM CONFIGALCADA C WHERE C.NRCONFIGALCADA = :NRCONFIGALCADA";
        $params = array(
            'NRCONFIGALCADA' => $NRCONFIGALCADA,
        );
        
        return $this->supUtilities->crud($sql, $params)[0];
    }
    
    public function getIdSisDos()
    {
        $sql = "SELECT 
                    IDSISDOS
                FROM
                    PARAMGERAL";
                    
        $params = array();
        
        return $this->supUtilities->crud($sql, $params)[0];
    }
    
    /********************
     *      UTILS       *
     * ******************/
     
    /**
     * Utiliza a row passada para criar um array de parâmetros
     * contendo os valores equivalentes ao Tipo da Alçada (preenchidos através da row)
     * e os que não fazem parte do tipo selecionado (preenchidos com string vazia).
     * 
     * @return Array contendo todos os parâmetros preenchidos.
     */
    public function criarParametrosConfiguracaoAlcada($row, $isOperacaoInclusao = true)
    {
        $parameters = array_fill_keys(self::NOME_PARAMETROS_CONFIG_ALCADA, 'DEFAULT');
        
        foreach ($parameters as $key => $value) {
            if (array_key_exists($key, $row) && !empty($row[$key])) {
                $parameters[$key] = $row[$key];
            }
        }
        
        if ($isOperacaoInclusao) {
            $parameters['NRCONFIGALCADA'] = floatval($this->supUtilities->generateCode('NRCONFIGALCADA'));
            $parameters['NRORG']          = $this->environment->getNrOrg();
            $parameters['NRORGINCLUSAO']  = $this->environment->getNrOrg();
            $parameters['CDOPERINCLUSAO'] = $this->environment->getCdOperador();
            self::defineDataComSysdate($parameters, self::DATAS_COM_SYSDATE_INSERT);
            $parameters = self::limparArray($parameters);
        } else {
            // Retorna o NRCONFIGALCADA vazio no update para que os parâmetros do SET sejam montados corretamente
            $parameters['NRCONFIGALCADA'] = '';
            self::defineDataComSysdate($parameters, self::DATAS_COM_SYSDATE_UPDATE);
        }
        
        $parameters['NRORGULTATU']    = $this->environment->getNrOrg();
        $parameters['CDOPERULTATU']   = $this->environment->getCdOperador();
        if(isset($row['IDTIPOVRAPROV'])){
            $parameters['IDTIPOVRAPROV']  = $row['IDTIPOVRAPROV'];
        }
        
        return $parameters;
    }
    
    /**
     * Elimina do array as chaves que não possuem valor (string em branco).
     * 
     * @return Array contendo somente os elementos não vazios.
     */
    public function limparArray($array)
    {
        return array_filter($array);
    }
    
    public function getNomeColunasInsertFormatado($parameters)
    {
        return implode(', ', array_keys($parameters));
    }
    
    public function getValorParametrosInsertFormatado($parameters)
    {
        return implode(', ', array_map(function($name, $value) {
            return self::getValorParametro($name, $value);
        }, array_keys($parameters), $parameters));
    }
    
    public function defineDataComSysdate(&$parameters, $datasQueUsamSysDate)
    {
        foreach ($datasQueUsamSysDate as $key) {
            $parameters[$key] = 'SYSDATE';
        }
    }
    
    public function removeSysdate($parameters, $datasQueUsamSysDate)
    {
        foreach ($datasQueUsamSysDate as $key) {
            unset($parameters[$key]);
        }
        
        return $parameters;
    }
    
    public function removeDefault($parameters)
    {
        $newParameters = array();
        
        foreach ($parameters as $key => $value) {
            if ($value != 'DEFAULT') {
                $newParameters[$key] = $value;
            }
        }
        
        return $newParameters;
    }
    
    public function getColunaAndValorParametrosUpdateFormatado($parameters)
    {
        return implode(', ', array_map(function($name, $value) {
            return $name . ' = ' . self::getValorParametro($name, $value);
        }, array_keys($parameters), $parameters));
    }
    
    public function getValorParametro($nome, $valor)
    {
        $retornarProprioValor = array(
            'SYSDATE',
            'DEFAULT',
        );
        
        return in_array($valor, $retornarProprioValor) ? $valor : ':' . $nome;
    }
    
    public function retornaOperadoresAlcada($nrconfigalcada, $cdfilial, $cdoperador, $cdoperrelacap)
    {
        $sql = "SELECT AF.CDOPERADOR, O.DSEMAILOPER FROM ALCADAGERFILI AF INNER JOIN OPERADOR O ON AF.CDOPERADOR = O.CDOPERADOR WHERE AF.NRCONFIGALCADA = :NRCONFIGALCADA AND((INSTR(:CDFILIAL, AF.CDFILIAL) <> 0) OR :CDFILIAL = 'T') AND ((INSTR(:CDOPERADOR, AF.CDOPERADOR) <> 0) OR :CDOPERADOR = 'T') AND (:CDOPERRELACAP = 'T' OR :CDOPERRELACAP = '1') UNION SELECT AO.CDOPERRELACAP AS CDOPERADOR, OP.DSEMAILOPER FROM ALCADAGEROPER AO INNER JOIN OPERADOR OP ON AO.CDOPERRELACAP = OP.CDOPERADOR WHERE NRCONFIGALCADA = :NRCONFIGALCADA AND ((INSTR(:CDFILIAL, AO.CDFILIAL) <> 0) OR :CDFILIAL = 'T') AND ((INSTR(:CDOPERADOR, AO.CDOPERADOR) <> 0) OR :CDOPERADOR = 'T') AND ((INSTR(:CDOPERRELACAP, AO.CDOPERRELACAP) <> 0) OR :CDOPERRELACAP = 'T')";
        
        $params = array(
            "NRCONFIGALCADA" => $nrconfigalcada,
            "CDFILIAL"       => $cdfilial,
            "CDOPERADOR"     => $cdoperador,
            "CDOPERRELACAP"  => $cdoperrelacap,
        );
        
        $operadores = $this->supUtilities->crud($sql, $params);
        
        return count($operadores) ? $operadores : false;
    }
    
    private function getDiretorioLog()
    {
        $directory = explode('/', __DIR__);
        array_pop($directory);
        $directory = implode('/', $directory) . '/log';
        
        return $directory;
    }
    
    private function getNomeArquivo($dataAtual, $numeroConfigAlcada)
    {
        return $dataAtual->format('d-m-Y') . "_" . $numeroConfigAlcada . "_LOG_ALTERACOES_CONFIG_ALCADA.txt";
    }
    
    private function getCaminhoCompletoArquivoLog($diretorio, $nomeArquivo)
    {
        return $diretorio . "/" . $nomeArquivo;
    }
    
    private function getLinhaLogFormatada($dataAtual, $nomePropriedade, $fraseDeAlteracao)
    {
        return "[" . $dataAtual->format('d-m-Y H:i:s') . " " . $this->environment->getCdOperador() . "] Propriedade " . $nomePropriedade . " " . $fraseDeAlteracao . "\n";
    }
}
